library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;


entity fpga_interface is
	port
	(
		--Global signals
		reset_n: in std_logic;
		fsm_clk : in std_logic;
		
		--FPGA interface
		max_csn: in std_logic;
		max_wen: in std_logic;
		max_oen: in std_logic;
		address: in std_logic_vector(4 downto 0);
		data: inout std_logic_vector(15 downto 0);
		max_ben : in std_logic_vector(3 downto 0); -- byte write enable signal
		
		--dipswitch/PFL interface
		dsw_pagesel: in std_logic_vector(2 downto 0);
		pfl_pagesel: out std_logic_vector(2 downto 0);
		srst_out : out std_logic;
		
		-- power and temp data
		ave_v, ave_a, ave_w : in std_logic_vector(15 downto 0);
		max_v, max_a, max_w : in std_logic_vector(15 downto 0);
		min_v, min_a, min_w : in std_logic_vector(15 downto 0);
		temp_data_r : in std_logic_vector(15 downto 0);
		temp_data_l : in std_logic_vector(15 downto 0);
		volt_data : in std_logic_vector(15 downto 0);
		fsd_req : in std_logic;
		
		-- clk control
		clk155_en : out std_logic;
		clk1_ce : out std_logic;
		clk2_ce : out std_logic;
		clk_sel : in std_logic;
		clk_enable : in std_logic;
		
		-- board setting
		sram_mode : out std_logic;
		sram_zz : out std_logic;
		hsma_orsntn : in std_logic;
		hsmb_orsntn : in std_logic;
		MAX_VER : in std_logic_vector(7 downto 0);
		
		-- OSC control
		CLK1_PR : out std_logic_vector(1 downto 0);
		CLK1_OD : out std_logic_vector(2 downto 0);
		CLK2_PR : out std_logic_vector(1 downto 0);
		CLK2_OD : out std_logic_vector(2 downto 0);
		clk1_rstn : out std_logic;
		clk2_rstn : out std_logic;
		
		system_clk : in std_logic
				
	);

end fpga_interface;


architecture rtl of fpga_interface is
	
	constant REGFILE_TOP: integer := 4;
	type regfile_t is array(1 to REGFILE_TOP) of std_logic_vector(15 downto 0);
	signal regfile: regfile_t;
	
	alias pss is regfile(2)(3 downto 0);
	alias pso is regfile(2)(4);
	alias psr is regfile(2)(10 downto 8);
	alias srst is regfile(2)(11);
	alias osc_u26 is regfile(2)(13 downto 12);
	alias osc_u30 is regfile(2)(15 downto 14);
		
	signal address_int : std_logic_vector(4 downto 0);
	signal srst_d1, srst_d2 : std_logic;
	
	signal reset_count : integer range 0 to 15;
	
	signal clk_reset_counter : integer range 0 to 31;
	signal osc_u26_old : std_logic_vector(1 downto 0);
	signal osc_u30_old : std_logic_vector(1 downto 0);
	signal clk_reset : std_logic;
begin
		
---------------------------------------------------------------------------------------------------------
-- FPGA interface logic
---------------------------------------------------------------------------------------------------------
	
	prc_regfile:
	process(reset_n,fsm_clk, regfile)begin	
		
		-------------------------------------------------
		--Reset clause
		-------------------------------------------------
		if(reset_n = '0')then
			pso		<= '1';
			srst <= '1';
			psr <= "111";
			regfile(3) <= (others => '1');
			regfile(4) <= "0000111100111100";
			regfile(2)(15 downto 12) <= "0001";
			regfile(2)(7 downto 5) <= "000";
			
		-------------------------------------------------
		--Operating clause
		-------------------------------------------------
		elsif(fsm_clk'event and fsm_clk = '1')then
			if(max_csn = '0')then
				address_int <= address;
				--Read clause
				if(max_wen = not('0') and max_oen = '0')then
				--Write clause
				elsif(max_wen = '0' and max_oen = not('0'))then
					case address is
						when "00010" => 
							if(max_ben(0) = '0')then
							end if;							
							if(max_ben(1) = '0')then
								regfile(2)(10 downto 8)	<= data(10 downto 8); -- PSR
							end if;
							if(max_ben(2) = '0')then
								regfile(2)(4)<= data(4); -- PSO
							end if;
							if(max_ben(3) = '0')then
								regfile(2)(15 downto 11) <= data(15 downto 11);
							end if;
						when "00011" => regfile(1) <= data;
						when "00100" => 
							if(max_ben(0) = '0')then
								regfile(3)(5 downto 0) <= data(5 downto 0);
							end if;							
						when "00101" => 
							if(max_ben(0) = '0')then
								regfile(4)(5 downto 0) <= data(5 downto 0);
							end if;							
							if(max_ben(1) = '0')then
								regfile(4)(12 downto 8) <= data(12 downto 8);
							end if;
							if(max_ben(2) = '0')then
							end if;
							if(max_ben(3) = '0')then
							end if;
						when others => null;
					end case;
				end if;		
			else
				srst <= '1'; -- need to clear the re-config request bit
			end if;
		end if;
	end process;

	process(max_oen, address_int, fsd_req, fsm_clk)begin
		if(max_oen = '1' or fsd_req = '1')then
			data <= (others => 'Z');
		elsif(fsm_clk'event and fsm_clk = '1')then
--		else
			case address_int is
				when "00000" => data <= x"9F50";  -- 0xA144 is the serial number of the S4PCIe dev kit.
				when "00001" => data <= x"00" & MAX_VER;  -- MAX-II program Rev
				when "00010" => data <= regfile(2); -- control reg RSU
				when "00011" => data <= regfile(1); -- reserved
				when "00100" => data(2 downto 0) <= regfile(3)(2 downto 0); -- clk control reg
								data(9 downto 8) <= clk_enable & clk_sel;
								data(7 downto 3) <= "00000";
								data(15 downto 10) <= "000000";
				when "00101" => data(7 downto 0) <= hsmb_orsntn & hsma_orsntn & regfile(4)(5 downto 0); -- board control reg
								data(12 downto 8) <= regfile(4)(12 downto 8);
								data(15 downto 11) <= (others => '0');
				when "00110" => data(15 downto 0) <= ave_v;
				when "00111" => data(15 downto 0) <= ave_a;
				when "01000" => data(15 downto 0) <= ave_w;
				when "01001" => data(15 downto 0) <= max_v;
				when "01010" => data(15 downto 0) <= max_a;
				when "01011" => data(15 downto 0) <= max_w;
				when "01100" => data(15 downto 0) <= min_v;
				when "01101" => data(15 downto 0) <= min_a;
				when "01110" => data(15 downto 0) <= min_w;
				when "10000" => data(15 downto 0) <= temp_data_r;
				when "10001" => data(15 downto 0) <= temp_data_l;
				when "10010" => data(15 downto 0) <= volt_data;
				when others => null;
			end case;
		end if;
	end process;

---------------------------------------------------------------------------------------------------------
--dipswitch/PFL interface
---------------------------------------------------------------------------------------------------------
	pss(2 downto 0) <= dsw_pagesel;
	pss(3) <= '0';

	prc_pfl:
	process(reset_n, psr, pso, dsw_pagesel, regfile)
	begin
		if(reset_n = '0')then
			pfl_pagesel <= dsw_pagesel(2 downto 0);	
		else
			if(pso = '1')then
				pfl_pagesel  <= dsw_pagesel(2 downto 0);			
			else
				pfl_pagesel  <= psr;				
			end if;
		end if;
	end process;		

	process(osc_u26)begin
		case osc_u26 is
		when "00" =>
			CLK2_PR <= "00";
			CLK2_OD <= "101";
		when "01" =>
			CLK2_PR <= "11";
			CLK2_OD <= "011";
		when "10" =>
			CLK2_PR <= "00";
			CLK2_OD <= "011";
		when "11" =>
			CLK2_PR <= "10";
			CLK2_OD <= "011";
		end case;
	end process;

	process(osc_u30)begin
		case osc_u30 is
		when "00" =>
			CLK1_PR <= "00";
			CLK1_OD <= "101";
		when "01" =>
			CLK1_PR <= "11";
			CLK1_OD <= "011";
		when "10" =>
			CLK1_PR <= "00";
			CLK1_OD <= "011";
		when "11" =>
			CLK1_PR <= "10";
			CLK1_OD <= "011";
		end case;
	end process;

	
	process(reset_n, srst, fsm_clk)begin
		if(reset_n = '0')then
			reset_count <= 15;
		elsif(srst = '0')then
			reset_count <= 0;
		elsif(fsm_clk'event and fsm_clk = '1')then
			if(reset_count = 15)then
				reset_count <= reset_count;
			else
				reset_count <= reset_count + 1;
			end if;
		end if;
	end process;

	process(reset_n, fsm_clk)begin
		if(reset_n = '0')then
			srst_out <= '1';	
		elsif(fsm_clk'event and fsm_clk = '1')then
			if(reset_count = 15)then
				srst_out <= '1';
			else
				srst_out <= '0';			
			end if;
		end if;
	end process;



	process(reset_n, clk_enable, regfile)begin
		if(reset_n = '0')then
			clk155_en <= '1'; 
--			clk1_ce <= '0';
--			clk2_ce <= '0';
		elsif(clk_enable = '1')then
			clk155_en <= '0'; 
--			clk1_ce <= '0';
--			clk2_ce <= '0';
		else
			clk155_en <= regfile(3)(0); 
--			clk1_ce <= regfile(3)(1);
--			clk2_ce <= regfile(3)(2);
		end if;
	end process;


	process(reset_n, clk_enable, regfile, clk_reset, system_clk)begin
		if(reset_n = '0')then
			clk1_ce <= '0';
			clk2_ce <= '0';
			clk1_rstn <= '0';
			clk2_rstn <= '0';
		elsif(system_clk'event and system_clk = '1')then
			if(clk_reset = '0')then
				clk1_ce <= '0';
				clk2_ce <= '0';
				clk1_rstn <= '0';
				clk2_rstn <= '0';		
			elsif(clk_enable = '1')then
				clk1_ce <= '0';
				clk2_ce <= '0';
				clk1_rstn <= '0';
				clk2_rstn <= '0';
			else
				clk1_ce <= regfile(3)(1);
				clk2_ce <= regfile(3)(2);
				clk1_rstn <= '1';
				clk2_rstn <= '1';
			end if;
		end if;
	end process;

	process(system_clk)begin
		if(system_clk'event and system_clk = '1')then
			if(osc_u26_old /= osc_u26)then
				clk_reset <= '0';
			elsif(osc_u30_old /= osc_u30)then
				clk_reset <= '0';			
			else
				clk_reset <= '1';
			end if;
			osc_u26_old <= osc_u26;
			osc_u30_old <= osc_u30;
		end if;
	end process;


	process(reset_n, regfile)begin
		if(reset_n = '0')then
			sram_mode <= '0';
			sram_zz <= '0';
--			sdi_tx_en <= '1';
--			sdi_rx_bypass <= '1';
--			sdi_rx_en <= 'Z'; -- this is Read Only
--			fan_force_on <= '1';
--			fan_speed <= "1111";
--			fan_speed_overwrite <= '0';
		else
			sram_mode <= regfile(4)(0);
			sram_zz <= regfile(4)(1);
--			sdi_tx_en <= regfile(4)(2);
--			sdi_rx_bypass <= regfile(4)(3);
--			sdi_rx_en <= 'Z'; -- this is Read Only
--			fan_force_on <= regfile(4)(5);
--			fan_speed(3 downto 0) <= regfile(4)(11 downto 8);
--			fan_speed_overwrite <= regfile(4)(12);
		end if;
	end process;

end rtl;

